function [Xg,Fg,ranks,flag,cdist1]=SelRank(X,U,Np,d,m,type,obj)
% Syntax: [Xg,Fg,ranks,flag,crdist1,idist1,cdist1]=SelRank(X,U,Np,d,m,type,obj)
% This function performs novel selection and ranking for IDEMO.
% Input - 
%   X: Parent population 
%   U: Trial population
%   Np: Scalar value defining no. of candidate solution in a population
%   d: Scalar value for the dimension of each encoded candidate vector 
%      (total number of decision variables)
%   m: Scalar value for total number of objectives
%   type: Type of the test problem (DTLZ used in this work)
%   obj: indices of objectives in the objective set under consideration
% Output - 
%   Xg: Population in decision space for the next generation 
%       (matrix order Np x d)
%   Fg: Population in function/objective space for the next generation 
%       (matrix order Np x d)
%   ranks: ranks corresponding to each candidate in population
%          (matrix order Np x 2) col 1: non-dominated sorting, col 2: Dcomb
%   flag: set if new candidate is found, reset if no new candidate appears
%   cdist1: value of Dcomb corresponding to each candidate

Xg=zeros(Np,d);
if strcmp(type(1:4),'DTLZ')
    Fp=evalfit(type,X(:,1:d-1),d-1,m); % last column is for weights
    Fc=evalfit(type,U(:,1:d-1),d-1,m); % doesnot play any part in fitness eval
else
    disp('Type of problem: unrecognized.\n')
end    
Fg=zeros(Np,m); % Initialize F for merged population
ranks=zeros(Np,2);
newchild=zeros(Np,1); % Intialize for new candidates which will be merged
flag=0; % assume there is no new candidate
%% Selection
for i=1:Np
    if Pdominates(Fp(i,obj),Fc(i,obj))==1 % if parent candidate dominates
        Xg(i,:)=X(i,:);
        Fg(i,:)=Fp(i,:);
    else % if parent candidate doesnot dominate
        Xg(i,:)=U(i,:);
        Fg(i,:)=Fc(i,:);
        newchild(i,1)=1; % there is a new child for merged population
    end
end

Ftotal=[Fp;Fg(logical(newchild),:)]; % merge population
Xtotal=[X;Xg(logical(newchild),:)]; % merge corresponding objectives
% Initialize labels to differentiate old and new candidates
checkpop(1:Np,1)=zeros(Np,1); 
Ntotal=size(Ftotal,1);
if size(Ftotal,1)>Np % if new child is there, set flag
    flag=1;
    checkpop(Np+1:Ntotal,1)=ones(Ntotal-Np,1); % label new candidates
end

%% Ranking
Start=1;
r=1;
while(1)
    % wrt Pareto dominates
    nd=ones(size(Ftotal,1),1); % initial assumption all are non-dominated
    for i=1:size(Ftotal,1)
        for j=1:size(Ftotal,1)
            stat=Pdominates(Ftotal(j,obj),Ftotal(i,obj));
            if stat==1 % if any solutions dominates i-th vector
                nd(i,1)=0; % it is dominated
                break;
            end
        end
    end
    [nd,ind1]=sort(nd,'descend'); % keep the non-dominated solutions earlier
    N1=size(nd(nd==1,:),1);
    % Rearrange Ftotal, Xtotal, checkpop
    Ftotal=Ftotal(ind1,:);
    Xtotal=Xtotal(ind1,:);
    checkpop=checkpop(ind1,:);
    % Select on lowest rank of solutions (X1, F1, cpop1)
    F1=Ftotal(1:N1,:);
    X1=Xtotal(1:N1,:);
    cpop1=checkpop(1:N1,:);
    
    % Rank wrt to Dcomb: combination of crowding distance and distance from
    % Ideal Point
    IdealP=zeros(1,m);
    [X1,F1,cpop1,cdist1]=combrank(X1,F1,cpop1,N1,obj,IdealP);

    Amt=N1;
    Last=Start+Amt-1;
    % Novel Elitist Selection
    % If R1 >= 0.75Np and Rrest >= 0.25Np
    %   Include 0.75Np of R1 and remaining from Rrest
    % If R1 >= 0.75Np and Rrest < 0.25Np
    %   Include Rrest and remaning from R1 to form Np
    % If R1 < 0.75Np and Rrest >= 0.25Np
    %   Include R1 and remaining from Rrest (tackled by default)
    % If R1 < 0.75Np and Rrest < 0.25Np
    %   Not possible as R1+Rrest>=Np
    if r==1 % Select indices for R1 solutions
        Amto=size(Ftotal,1)-Amt;
        if Amto>=(0.25*Np) && Amt>=(0.75*Np)
            Last=(0.75*Np);
            Amt=(0.75*Np)-Start+1;
        elseif Amto<(0.25*Np) && Amt>=(0.75*Np)
            Last=Np-Amto;
            Amt=(Np-Amto)-Start+1;
        end
    end
    
    if r~=1 && Last>=Np % Select indices for Rrest solutions
        Last=Np;
        Amt=Np-Start+1;
        flag=2; % if this is last rank
    end
    % Trim population based on selected indices
    Fg(Start:Last,:)=F1(1:Amt,:);
    Xg(Start:Last,:)=X1(1:Amt,:);
    checkpopfinal(Start:Last,:)=cpop1(1:Amt,:);
    ranks(Start:Last,1)=r;
    ranks(Start:Last,2)=cdist1(1:Amt,:);

    if flag==2 % if last rank
        for k=1:Np
            if checkpopfinal(k,:)==1 % check for new solution
                flag=1; % set flag if new solution is there
                break;
            end
        end
        break;
    else % if rank is not last
        % remove population already selected from merged population
        Ftotal=Ftotal(N1+1:end,:);
        Xtotal=Xtotal(N1+1:end,:);
        checkpop=checkpop(N1+1:end,:);
        Start=Last+1; % reset starting index
        r=r+1; % increment rank of population
    end
end
end