function [Fx]=evalfit(Type,x,num_var,num_objs,alpha)
% Syntax: [Fx]=evalfit(Type,x,num_var,num_objs,alpha)
% This function evaluates objectives corresponding to each candidate of the
% population. Type indicates the problem kind. DTLZ 1-4 have been encoded.
% Input - 
%   Type: type of the DTLZ test problem
%   x: Population at generation G (matrix order Np x d)
%   num_var: Size of candidate vectors (#columns of population matrix)
%   num_objs: No. of objectives in the full objective set
%   alpha: Vector of indices labelling the objectives under consideration
% Output - 
%   Fx: Objective vectors corresponding to each candidate in x
%       (matrix order Np x d)

Popsize=size(x,1);
if(nargin==4)
    alpha=(1:num_objs)';
end

iter=size(alpha,1);
if(iter>num_objs)
    error(message('Invalid Input for alpha'));
end
if(num_var~=size(x,2))
    error(message('Invalid Input Dimension'));
end
xbar=num_var-num_objs+1;
gstart=num_var-xbar+1;
Fx=zeros(Popsize,num_objs);
switch Type
    case 'DTLZ1'
        for i=1:Popsize
            %%Evaluation of the G(x) Function for a Particular member of the Population
            Gx=xbar;
            for j=gstart:num_var
               Gx=Gx+(x(i,j)-0.5)*(x(i,j)-0.5)-cos(20*pi()*(x(i,j)-0.5));
            end
            Gx=1+100*Gx;
            
            %%Evaluation of the functions for a particular member of the population
            %Fx(i,1)=0.5*prod(x(i,1:num_objs-1))*Gx;
            for j=1:iter
                k=alpha(j,1);
                Fx(i,k)=0.5*prod(x(i,1:num_objs-k))*Gx;
                if(k~=1)
                     Fx(i,k)=Fx(i,k)*(1-x(i,num_objs-k+1));
                end
                if(k==num_objs)
                    Fx(i,k)=0.5*(1-x(i,1))*Gx;
                end
                if (Fx(i,j) < 1e-10)
                      Fx(i,j)=0;
                end
            end
        end
    case 'DTLZ2'
        for i=1:Popsize
            %%Evaluation of the G(x) Function for a Particular member of the Population
            Gx=sum((x(i,gstart:num_var)-0.5).^2);
            
            %%Evaluation of the functions for a particular member of the population
            for j=1:iter
                k=alpha(j,1);
                Fx(i,k)=prod(cos(0.5*pi()*x(i,1:num_objs-k)))*(1+Gx);
                if(k~=1)
                   Fx(i,k)=Fx(i,k)*sin(0.5*pi()*x(i,num_objs-k+1));
                end
                if (Fx(i,j) < 1e-10)
                      Fx(i,j)=0;
                end
            end
        end
    case 'DTLZ4'
        for i=1:Popsize
            %%Evaluation of the G(x) Function for a Particular member of the Population
            Gx=sum((x(i,gstart:num_var)-0.5).^2);
            
            %%Evaluation of the functions for a particular member of the population
            for j=1:iter
                k=alpha(j,1);
                Fx(i,k)=prod(cos(0.5*pi()*x(i,1:num_objs-k).^100))*(1+Gx);
                if(k~=1)
                   Fx(i,k)=Fx(i,k)*sin(0.5*pi()*x(i,num_objs-k+1).^100);
                end
                if (Fx(i,j) < 1e-10)
                      Fx(i,j)=0;
                end
            end
        end
    case 'DTLZ3'
        for i=1:Popsize
            %%Evaluation of the G(x) Function for a Particular member of the Population
            Gx=xbar;
            for j=gstart:num_var
               Gx=Gx+(x(i,j)-0.5)*(x(i,j)-0.5)-cos(20*pi()*(x(i,j)-0.5));
            end
            Gx=1+100*Gx;
            
            %%Evaluation of the functions for a particular member of the population
            for j=1:iter
                k=alpha(j,1);
                Fx(i,k)=prod(cos(0.5*pi()*x(i,1:num_objs-k)))*(Gx);
                if(k~=1)
                   Fx(i,k)=Fx(i,k)*sin(0.5*pi()*x(i,num_objs-k+1));
                end
                if (Fx(i,j) < 1e-10)
                      Fx(i,j)=0;
                end
            end
        end
    otherwise
        warning('Unexpected Function type. plese enter a valid choice');
        return
end
end